@@ -4,7 +4,11 @@
 #include <QMainWindow>
 #include <QNetworkAccessManager>
 #include <QDebug>
-#include <QListWidgetItem>
+#include <QFileDialog>
+#ifdef Q_WS_MAEMO_5
+    #include <QMaemo5InformationBox>
+#endif
+#include <QMessageBox>
 #include "widgets/photoitem.h"
 #include "imageuploaddialog.h"
 #include "qfacebook.h"
@@ -36,14 +40,21 @@ private:
     QNetworkAccessManager *nam;
     QString albumCount;
     QList<PhotoItem*> photos;
+    QStringList sourceLinks;
+    QDialog *progressDialog;
     int column;
     int row;
+    int m_imagesCount;
+    int m_finishedDownloads;
+    QString m_downloadLocation;
 
 private slots:
     void updateAlbum();
     void onReplyReceived();
     void uploadPhoto();
     void onPhotoItemClicked(PhotoItem*);
+    void onDownloadFinished();
+    void downloadAllPhotos();
 };
 
 #endif // ALBUMWINDOW_H
