@@ -85,5 +85,9 @@ void EventsWindow::onReplyReceived()
 void EventsWindow::createEvent()
 {
     NewEventDialog *dialog = new NewEventDialog(this, accessToken, qfacebook);
+#ifdef Q_WS_S60
+    dialog->showMaximized();
+#else
     dialog->show();
+#endif
 }
