@@ -5,6 +5,7 @@ FriendsWindow::FriendsWindow(QWidget *parent, QString accessToken) :
     QMainWindow(parent),
     ui(new Ui::FriendsWindow),
     token(accessToken),
+    qfacebook(new QFacebook(accessToken, this)),
     nam(new QNetworkAccessManager())
 {
     ui->setupUi(this);
@@ -18,6 +19,8 @@ FriendsWindow::FriendsWindow(QWidget *parent, QString accessToken) :
 #else
     ui->searchHideButton->setText("X");
 #endif
+    m_isDialog = false;
+    m_shouldFetchAvatars = QSettings().value("friends/fetch-avatars", true).toBool();
     connect(nam, SIGNAL(finished(QNetworkReply*)), this, SLOT(onAvatarReplyFinished(QNetworkReply*)));
     connect(ui->friendsList, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(onFriendClicked(QListWidgetItem*)));
     connect(ui->actionRefresh, SIGNAL(triggered()), this, SLOT(updateFriends()));
@@ -42,7 +45,12 @@ void FriendsWindow::browseId(QString id)
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 #endif
     this->idToBrowse = id;
-    qfacebook = new QFacebook(this->token, this);
+
+    if (id == "me" && QSettings().contains("friends/list")) {
+        qDebug() << "FriendsWindow: Cache exists, loading from there...";
+        m_friendsList = QSettings().value("friends/list");
+        this->parseData(m_friendsList);
+    }
 
     this->reply = qfacebook->getConnections(this->idToBrowse, "friends");
     if (this->reply)
@@ -59,32 +67,51 @@ void FriendsWindow::onReplyFinished()
         return;
     }
 
+    QVariant jsonData = this->reply->data();
+    if (idToBrowse == "me") {
+        if (m_friendsList == jsonData) {
+            qDebug() << "FriendsWindow: Friends list cache seems up to date...";
+            return;
+        } else {
+            qDebug() << "FriendsWindow: Either there's no cache, or you gained or lost a friend, updating cache...";
+            QSettings().setValue("friends/list", jsonData);
+        }
+    }
+
+    this->parseData(jsonData);
+}
+
+void FriendsWindow::parseData(QVariant jsonData)
+{
     ui->friendsList->clear();
 
-    QVariant jsonData = this->reply->data();
-    QVariantList listData;
-    listData = jsonData.toMap().value("data").toList();
-    foreach(jsonData, listData) {
-        QListWidgetItem *item = new QListWidgetItem(ui->friendsList);
-        item->setText(jsonData.toMap().value("name").toString());
-        item->setData(Qt::UserRole, jsonData.toMap().value("id").toString());
-        Friend *user = new Friend(ui->friendsList, jsonData.toMap().value("id").toString());
-        connect(user, SIGNAL(avatarProcessed(QString,QPixmap)), this, SLOT(onAvatarProcessed(QString,QPixmap)));
-        QString avatarUrl = QString("http://graph.facebook.com/%1/picture").arg(jsonData.toMap().value("id").toString());
-        QNetworkReply *reply =
-                nam->get(QNetworkRequest(avatarUrl));
-        connect(reply, SIGNAL(finished()), user, SLOT(avatarReceived()));
-    }
+    QThread *thread = new QThread;
+    FriendParser *parser = new FriendParser(jsonData, m_isDialog, m_shouldFetchAvatars);
+    parser->moveToThread(thread);
+
+    connect(thread, SIGNAL(started()), parser, SLOT(start()));
+    connect(thread, SIGNAL(finished()), parser, SLOT(deleteLater()));
+    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
+    connect(parser, SIGNAL(avatarReceived(QString,QImage)), this, SLOT(onAvatarProcessed(QString,QImage)));
+    connect(parser, SIGNAL(itemParsed(QListWidgetItem*)), this, SLOT(onItemParsed(QListWidgetItem*)));
+
+    thread->start();
+
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
 #endif
 }
 
-void FriendsWindow::onAvatarProcessed(QString id, QPixmap pixmap)
+void FriendsWindow::onItemParsed(QListWidgetItem *item)
+{
+    ui->friendsList->addItem(item);
+}
+
+void FriendsWindow::onAvatarProcessed(QString id, QImage image)
 {
     for (int i = 0; i < ui->friendsList->count(); i++) {
         if (ui->friendsList->item(i)->data(Qt::UserRole).toString() == id)
-            ui->friendsList->item(i)->setIcon(QIcon(pixmap));
+            ui->friendsList->item(i)->setIcon(QIcon(QPixmap::fromImage(image)));
     }
 }
 
@@ -105,8 +132,13 @@ void FriendsWindow::requestRecepientsList()
     connect(doneButton, SIGNAL(clicked()), this, SLOT(onRecepientsSelected()));
     ui->verticalLayout->addWidget(doneButton);
     this->setWindowFlags(Qt::Dialog);
+    m_isDialog = true;
     this->browseId("me");
+#ifdef Q_WS_S60
+    this->showMaximized();
+#else
     this->show();
+#endif
 }
 
 void FriendsWindow::requestFriend()
@@ -114,8 +146,13 @@ void FriendsWindow::requestFriend()
     disconnect(ui->friendsList, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(onFriendClicked(QListWidgetItem*)));
     connect(ui->friendsList, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(onRecepientsSelected()));
     this->setWindowFlags(Qt::Dialog);
+    m_isDialog = true;
     this->browseId("me");
+#ifdef Q_WS_S60
+    this->showMaximized();
+#else
     this->show();
+#endif
 }
 
 void FriendsWindow::onRecepientsSelected()
@@ -130,7 +167,11 @@ void FriendsWindow::onFriendClicked(QListWidgetItem *item)
     window->setAttribute(Qt::WA_DeleteOnClose);
     window->browseProfile(item->data(Qt::UserRole).toString());
     window->setWindowTitle(item->text());
+#ifdef Q_WS_S60
+    window->showMaximized();
+#else
     window->show();
+#endif
 }
 
 void FriendsWindow::keyReleaseEvent(QKeyEvent *e)
@@ -176,13 +217,75 @@ void Friend::avatarReceived()
     QUrl redir = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
     if (!redir.isEmpty()) {
         reply->deleteLater();
+        QString url = redir.toString();
+        int pos = url.lastIndexOf("/");
+        QString baseName = url.remove(0, pos);
+        baseName.remove("/");
+
+        QFile cachedImage(QDesktopServices::storageLocation(QDesktopServices::CacheLocation) + "/" + baseName);
+        if (cachedImage.exists()) {
+            cachedImage.open(QIODevice::ReadOnly);
+            QByteArray data = cachedImage.readAll();
+            cachedImage.close();
+            emit avatarProcessed(this->userId, QImage::fromData(data));
+            return;
+        }
         QNetworkAccessManager *nam = qobject_cast<QNetworkAccessManager*>(sender()->parent());
         QNetworkReply *reply = nam->get(QNetworkRequest(redir));
 
         connect(reply, SIGNAL(finished()), this, SLOT(avatarReceived()));
         return;
     }
-    emit avatarProcessed(this->userId, QPixmap::fromImage(QImage::fromData(reply->readAll())));
+    QByteArray data = reply->readAll();
+    emit avatarProcessed(this->userId, QImage::fromData(data));
+
+    QString url = reply->url().toString();
+    int pos = url.lastIndexOf("/");
+    QString baseName = url.remove(0, pos);
+    baseName.remove("/");
+    QDir tempDir(QDesktopServices::storageLocation(QDesktopServices::CacheLocation));
+    if (!tempDir.exists())
+        tempDir.mkpath(QDesktopServices::storageLocation(QDesktopServices::CacheLocation));
+
+    QString filename = QDesktopServices::storageLocation(QDesktopServices::CacheLocation) + "/" + baseName;
+    QFile cachedImage(filename);
+    cachedImage.open(QIODevice::WriteOnly);
+    cachedImage.write(data);
+    cachedImage.close();
+
     reply->deleteLater();
     this->deleteLater();
 }
+
+FriendParser::FriendParser(QVariant jsonData, bool isDialog, bool shouldFetchAvatars)
+{
+    nam = new QNetworkAccessManager(this);
+    m_isDialog = isDialog;
+    m_jsonData = jsonData;
+    m_shouldFetchAvatars = shouldFetchAvatars;
+}
+
+void FriendParser::start()
+{
+    QVariant jsonData = m_jsonData;
+    QVariantList listData;
+    listData = jsonData.toMap().value("data").toList();
+    foreach(jsonData, listData) {
+        QListWidgetItem *item = new QListWidgetItem();
+        item->setText(jsonData.toMap().value("name").toString());
+        item->setData(Qt::UserRole, jsonData.toMap().value("id").toString());
+        if ((m_isDialog && m_shouldFetchAvatars) || !m_isDialog) {
+            Friend *user = new Friend(0, jsonData.toMap().value("id").toString());
+            connect(user, SIGNAL(avatarProcessed(QString,QImage)), this, SLOT(onAvatarProcessed(QString,QImage)));
+            QString avatarUrl = QString("http://graph.facebook.com/%1/picture").arg(jsonData.toMap().value("id").toString());
+            QNetworkReply *reply = nam->get(QNetworkRequest(avatarUrl));
+            connect(reply, SIGNAL(finished()), user, SLOT(avatarReceived()));
+        }
+        emit itemParsed(item);
+    }
+}
+
+void FriendParser::onAvatarProcessed(QString id, QImage avatar)
+{
+    emit avatarReceived(id, avatar);
+}
