@@ -8,7 +8,9 @@
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
 #include <QKeyEvent>
+#include <QSettings>
 #include <QListWidgetItem>
+#include <QThread>
 #include "profilewindow.h"
 #include <qfacebook.h>
 
@@ -27,6 +29,7 @@ public:
     void setToken(QString token);
     void requestRecepientsList();
     void requestFriend();
+    QString m_cacheLocation;
 
 protected:
     void keyReleaseEvent(QKeyEvent *);
@@ -36,20 +39,50 @@ signals:
 
 private:
     Ui::FriendsWindow *ui;
+    QString token;
     QFacebook *qfacebook;
     QFacebookReply *reply;
     QString idToBrowse;
-    QString token;
     QNetworkAccessManager *nam;
+    bool m_isDialog;
+    bool m_shouldFetchAvatars;
+    void parseData(QVariant jsonData);
+    QVariant m_friendsList;
 
 private slots:
     void onReplyFinished();
     void onAvatarReplyFinished(QNetworkReply *reply);
-    void onAvatarProcessed(QString id, QPixmap pixmap);
+    void onAvatarProcessed(QString id, QImage image);
     void onRecepientsSelected();
     void onFriendClicked(QListWidgetItem*);
     void updateFriends();
     void onSearchTextChanged(QString);
+    void onItemParsed(QListWidgetItem*);
+};
+
+class FriendParser : public QObject
+{
+    Q_OBJECT
+
+public:
+    explicit FriendParser(QVariant jsonData, bool isDialog = false, bool shouldFetchAvatars = true);
+
+public slots:
+    void start();
+
+signals:
+    void finished();
+    void itemParsed(QListWidgetItem*);
+    void avatarReceived(QString id, QImage image);
+
+private:
+    QVariant m_jsonData;
+    QNetworkAccessManager *nam;
+    bool m_isDialog;
+    bool m_shouldFetchAvatars;
+
+private slots:
+    void onAvatarProcessed(QString id, QImage avatar);
 };
 
 class Friend : public QObject
@@ -61,7 +94,7 @@ public:
     ~Friend();
 
 signals:
-    void avatarProcessed(QString, QPixmap);
+    void avatarProcessed(QString, QImage);
 
 public slots:
     void avatarReceived();
