@@ -1,14 +1,22 @@
 #include <QtGui/QApplication>
+#include <QTime>
 #include "mainwindow.h"
 
 int main(int argc, char *argv[])
 {
     QApplication::setApplicationName("sociality");
     QApplication::setOrganizationName("MohammadAG");
-    QApplication::setApplicationVersion("0.1");
+    QApplication::setApplicationVersion("0.2");
+#ifdef Q_WS_MAEMO_5
+    QApplication::addLibraryPath("/opt/qtm12/imports");
+    QApplication::addLibraryPath("/opt/qtm12/lib");
+#endif
     QApplication a(argc, argv);
 
+    QTime t(0,0);
+    t.start();
     MainWindow w;
+    qDebug() << "Startup took" << t.elapsed() << "ms";
 #ifdef Q_WS_S60
     w.showMaximized();
 #else
