@@ -9,11 +9,20 @@
 #include <QPainter>
 #include <QMainWindow>
 #include <QPropertyAnimation>
+#include <QMessageBox>
+#include <QMenu>
+#include <QClipboard>
+#include <QSettings>
+#include <QDesktopServices>
+#include <QFile>
+#include <QDir>
 #ifdef Q_WS_MAEMO_5
     #include <QMaemo5InformationBox>
 #endif
 #include <qfacebook.h>
 
+#include "youtubeplayer.h"
+
 namespace Ui {
     class NewsFeedPost;
 }
@@ -42,6 +51,8 @@ public:
     void setMessageMode(bool isMessage);
     void setTimeFromString(QString timeAsString);
     void getPixmapFromUserId(QString userId);
+    void addLink(QString url, QString linkName, QString linkDescription, QString linkPixmap);
+    void setToName(QString name, QString id);
 
 protected:
     void paintEvent(QPaintEvent *);
@@ -67,16 +78,20 @@ private:
     QNetworkReply *deleteReply;
     QString id;
     QNetworkAccessManager *nam;
+    QNetworkReply *pictureReply;
     QString comments;
     QString likes;
     QString userId;
+    QString m_toUserId;
     bool isLikedByMe;
     bool isDeletable;
     bool isComment;
     bool m_pressed;
+    bool m_whiteMode;
 
 private slots:
     void avatarReceived();
+    void onPictureReceived();
     void iconReceived();
     void orientationChanged();
     void onLikeButtonClicked();
@@ -85,6 +100,10 @@ private slots:
     void onDeleteButtonClicked();
     void onDeleteReplyReceived();
     void onNameClicked();
+    void copyPost();
+    void showContextMenu(QPoint point);
+    void onToNameClicked();
+    void onLinkActivated(QString url);
 };
 
 #endif // NEWSFEEDPOST_H
