@@ -12,6 +12,7 @@ PhotoWindow::PhotoWindow(QWidget *parent, QString token) :
     ui->setupUi(this);
     setAttribute(Qt::WA_DeleteOnClose);
     connect(ui->photoLabel, SIGNAL(clickedPos(QPoint)), this, SLOT(onClicked(QPoint)));
+    connect(ui->actionDownload_photo, SIGNAL(triggered()), this, SLOT(downloadPhoto()));
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5StackedWindow);
 #endif
@@ -26,7 +27,7 @@ void PhotoWindow::showPhoto(QString photoId)
 {
     this->photoId = photoId;
     QString urlString = QString("https://graph.facebook.com/%1/picture&type=large&access_token=%2").arg(photoId, accessToken);
-    qDebug() << urlString;
+    m_photoUrl = urlString;
     this->reply = nam->get(QNetworkRequest(urlString));
     if (this->reply)
         connect(reply, SIGNAL(finished()), this, SLOT(onReplyReceived()));
@@ -36,7 +37,7 @@ void PhotoWindow::showPhotoFromId(QString id)
 {
     this->photoId = id;
     QString urlString = QString("https://graph.facebook.com/%1&type=large&access_token=%2").arg(photoId, accessToken);
-    qDebug() << urlString;
+    m_photoUrl = urlString;
     this->reply = nam->get(QNetworkRequest(urlString));
     if (this->reply)
         connect(reply, SIGNAL(finished()), this, SLOT(onReplyReceived()));
@@ -49,6 +50,7 @@ void PhotoWindow::setPixmap(QPixmap pixmap)
 
 void PhotoWindow::setPixmapFromUrl(QString url)
 {
+    m_photoUrl = url;
     this->reply = nam->get(QNetworkRequest(url));
     if (this->reply)
         connect(reply, SIGNAL(finished()), this, SLOT(onReplyReceived()));
@@ -125,3 +127,44 @@ void PhotoWindow::setTags(QVariantList tags)
 {
     m_tags = tags;
 }
+
+void PhotoWindow::downloadPhoto()
+{
+    if (m_photoUrl.isEmpty()) {
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this, tr("Unable to download: invalid URL"));
+#else
+        QMessageBox::warning(this, tr("Error"), tr("Unable to download: invalid URL"));
+#endif
+        return;
+    }
+
+    m_downloadLocation = QFileDialog::getSaveFileName(this, tr("Save image as"),
+                                                      QDir::homePath());
+    if (m_downloadLocation.isEmpty()) {
+        return;
+    }
+
+    setAttribute(Qt::WA_DeleteOnClose, false);
+
+    m_downloadReply = nam->get(QNetworkRequest(m_photoUrl));
+    connect(m_downloadReply, SIGNAL(finished()), this, SLOT(onDownloadFinished()));
+}
+
+void PhotoWindow::onDownloadFinished()
+{
+    QFile image(m_downloadLocation, this);
+    if (image.open(QFile::WriteOnly)) {
+        image.write(m_downloadReply->readAll());
+        image.close();
+    }
+
+#ifdef Q_WS_MAEMO_5
+    QMaemo5InformationBox::information(this, tr("Download complete"));
+#else
+    QMessageBox::information(this, tr("Success"), tr("Photo downloaded"));
+#endif
+    setAttribute(Qt::WA_DeleteOnClose, true);
+        if (!isVisible())
+            this->deleteLater();
+}
