@@ -4,6 +4,7 @@
 #include <QMainWindow>
 #include <QNetworkAccessManager>
 #include <QDebug>
+#include <QFileDialog>
 #include <qfacebook.h>
 
 namespace Ui {
@@ -33,15 +34,20 @@ private:
     QNetworkReply *reply;
     QString accessToken;
     QNetworkAccessManager *nam;
+    QNetworkReply *m_downloadReply;
     QString photoId;
     QString m_userId;
     QVariantList m_comments;
     QVariantList m_tags;
+    QString m_photoUrl;
+    QString m_downloadLocation;
     QList<QWidget*> commentsList;
 
 private slots:
     void onReplyReceived();
     void onClicked(QPoint);
+    void downloadPhoto();
+    void onDownloadFinished();
 };
 
 #endif // PHOTOWINDOW_H
