@@ -41,6 +41,8 @@ private slots:
     void onCommentReplyReceived();
     void onPostButtonClicked();
     void orientationChanged();
+    void onPostDestroyed(QObject*);
+    void onNameClicked(QString,QString);
 };
 
 #endif // SINGLEPOSTWINDOW_H
