@@ -1,5 +1,6 @@
 #include "placeitem.h"
 #include "ui_placeitem.h"
+#include "profilewindow.h"
 
 PlaceItem::PlaceItem(QWidget *parent, QString token, QNetworkAccessManager *nam) :
     QWidget(parent),
@@ -11,6 +12,14 @@ PlaceItem::PlaceItem(QWidget *parent, QString token, QNetworkAccessManager *nam)
     QPalette p = this->palette();
     p.setColor(QPalette::Foreground, p.highlight().color());
     ui->nameLabel->setPalette(p);
+
+    bool whiteMode = QSettings().value("main/white-mode", false).toBool();
+    if (whiteMode) {
+        p.setColor(QPalette::Foreground, Qt::black);
+        ui->locationLabel->setPalette(p);
+        ui->timeLabel->setPalette(p);
+    }
+    connect(ui->nameLabel, SIGNAL(clicked()), this, SLOT(onNameClicked()));
 }
 
 PlaceItem::~PlaceItem()
@@ -94,6 +103,18 @@ void PlaceItem::mouseReleaseEvent(QMouseEvent *ev)
     }
 }
 
+void PlaceItem::onNameClicked()
+{
+    ProfileWindow *profile = new ProfileWindow(this->parentWidget(), accessToken);
+    profile->browseProfile(m_userId);
+    profile->setWindowTitle(ui->nameLabel->text());
+#ifdef Q_WS_S60
+    profile->showMaximized();
+#else
+    profile->show();
+#endif
+}
+
 bool PlaceItem::isPressed()
 {
     return m_pressed;
